/*
  IR Detector 
  John Plocher, based on GEoff Bunza's DAPD, 
  modified by Seth Neumann to make timing consistent with 
  EOPD, cpOD, DCC_OD.  Modify to your heart's content!
*/

#include <Wire.h>
// #include <LiquidCrystal_I2C.h>
#include <elapsedMillis.h>

#define PIN_DET1                2 // pin from IR comparator
#define PIN_DET2                3 // pin from IR comparator
#define PIN_DET3                4 // pin from IR comparator
#define PIN_DET4                5 // pin from IR comparator

#define PIN_OUT1                6 // pin to feedback connector
#define PIN_OUT2                7 // pin to feedback connector
#define PIN_OUT3                8 // pin to feedback connector
#define PIN_OUT4                9 // pin to feedback connector

#define PIN_LED1                11 // pin to LED
#define PIN_LED2                12 // pin to LED
#define PIN_LED3                13 // pin to LED
#define PIN_LED4                A0 // pin to LED
#define PIN_LED_STATUS          10 // Status LED to Flash on and off ever 100mS

#define I2C_ADDR    0x27
#define STATUS_DELAY 500  // John likes 800

#define VERSION "v0.11"  // initial version, debug LEDS and basic functionality
#define DATE    "9-29-2019" // mods by SCN to get the outputs to conform to labels on board
#define NAME    "Quad IR Detector"

// LiquidCrystal_I2C  lcd(I2C_ADDR, 16,2);
elapsedMillis updateLCDWatchdog;

//#define DEBUG

#define HYSTERESIS 3000    // in mS units, 2000 = 3.0 seconds, to conform to DCC_OD, cpOD, EOPD
#define OCCUPIED LOW
#define EMPTY    ~OCCUPIED
int LED_state; 
elapsedMillis statusBlinkTime;

class Circuit {
public:
    Circuit(void) {};
    void init(int number, int det, int out, int led) {
        num = number;
        detPin = det;
        outPin = out;
        ledPin = led;
        
        detected = false;
        delaytime = 0;
        pinMode(ledPin,  OUTPUT);
        pinMode(outPin,  OUTPUT);
        pinMode(detPin,  INPUT);

        digitalWrite(outPin, !EMPTY);    // Leave the detection pin "off"
        digitalWrite(ledPin, EMPTY);    // Leave the LED "off"
};
    
    int check(void) {
        int trackstatus =  digitalRead(detPin);   // read state
        
        if (trackstatus) {           // something has been detected...
            delaytime = 0;  // expiration timer is reset every time detection is seen
            if (detected == false) { // newly triggered
#ifdef DEBUG
                Serial.print("ON  ");
                Serial.println(num, DEC);
#endif
            }
            detected = true;
        }
        if (detected) {
            if (delaytime < HYSTERESIS) {
                digitalWrite(outPin, !OCCUPIED); 
                digitalWrite(ledPin, OCCUPIED); 
            } else {
#ifdef DEBUG
                Serial.print("OFF "); 
                Serial.println(num, DEC);
#endif
                digitalWrite(outPin, !EMPTY); 
                digitalWrite(ledPin, EMPTY); 
                detected = false;
            }
        }
        return detected ? 1 : 0;
    };
    
private:  
    int num;
    elapsedMillis delaytime;
    int detected;
    int detPin;
    int outPin;
    int ledPin;
};

Circuit circuit[4];

void setup() {

/*
  lcd.init();               // initialize the lcd
  lcd.backlight();
  lcd.setCursor(0,  0);
  lcd.print(NAME);
  lcd.setCursor(0, 1);
  lcd.print(DATE);
  lcd.print(VERSION);
  */
  LED_state == LOW;
  updateLCDWatchdog = 0;

  Serial.begin (115200);
  Serial.println(NAME);
  Serial.println(DATE);
  Serial.println(VERSION);
  pinMode(PIN_LED_STATUS,  OUTPUT); 

  circuit[0].init(1,PIN_DET1, PIN_OUT1, PIN_LED1);
  circuit[1].init(2,PIN_DET2, PIN_OUT2, PIN_LED2);
  circuit[2].init(3,PIN_DET3, PIN_OUT3, PIN_LED3);
  circuit[3].init(4,PIN_DET4, PIN_OUT4, PIN_LED4);

 /* while (updateLCDWatchdog < 1000) {
    delay(10);
  }
  */
}

void loop() {
  
#ifdef DEBUG
    if (updateLCDWatchdog > 100) { // delay between checks to keep serial debug stream reasonable
        for (int x = 0; x < 4; x++) {
            circuit[x].check();
        }
        updateLCDWatchdog = 0;
    }
#else // no delay needed
    for (int x = 0; x < 4; x++) {
        circuit[x].check();
    if(statusBlinkTime > STATUS_DELAY) {
    LED_state = !LED_state;
    statusBlinkTime = 0;
    digitalWrite(PIN_LED_STATUS,LED_state);

    }    
   
    }
#endif
}


